function back2 = backproj (back, proj, theta)
% back2 = backproj(back, proj, theta)
%
%	This function back projects a single projection vector 'proj'
%  obtained at an angle 'theta' into a real space backprojection array 
%  'back' with interpolation between samples in the projection; it returns 
%  the modified backproject array.  The center of rotation is assumed to
%  be at the center of the backprojection array and the center of the 
%  projection vector.  The back projection array should be initialized 
%  to zero before backproj is called with the first projection vector.  
%  The angle is measured counterclockwise and zero corresponds to a vertical
%  (column) sum of the backprojection matrix; the image is assumed to
%  be viewed using the matlab image function which displays the lowest-
%  numbered row at the top of the image.
%
%     Here is a sample program using the backproj function, where nback 
%  is the desired size of the backprojection array and dangle is the
%  angle increment between projections.
%
%  [nang, nbin] = size(sinogram);
%  back = zeros(nback, nback);
%  for ang = 1:nang
%     back = backproj(back, sinogram(ang,:), (ang-1)*dangle);
%     end

%  This version of backprojection has been somewhat optimized for speed
%  and is thus somewhat difficult to understand;  the alternate version
%  backpro.m is slower but easier to understand.

%  Summary of variables used
%   x, y	Coordinates in backprojection array
%   u		Coordinate in projection array
%   iu		Integer part of u
%   fu		Fractional part of u
%   xc, yc	Center of backprojection array
%   uc		Center of projection array
%   sint, cost	Sine and cosine of theta

%  Find sizes and compute some useful parameters
[nx,ny] = size(back);
m = max(size(proj));
proj = [0; proj(:); 0; 0];
xc = (nx+1)/2.0;
yc = (ny+1)/2.0;
uc = (m+1)/2.0;
sint = sin (theta*pi/180.0);
cost = cos (theta*pi/180.0);

% Do the backprojection by columns of the backprojection array
back2 = back;
x = 1:nx;
for y = 1:ny;
   u = uc + (xc-x)*sint + (y-yc)*cost;
   iu = floor(u);
   fu = u - iu;
   iu = max(1, iu+1);
   iu = min(iu, m+2);
   back2(:,y) = back2(:,y) + (1.0-fu)'.*proj(iu) + fu'.*proj(iu+1);
   end
end
